package gregapi.cover;

import static gregapi.data.CS.ALL_SIDES_VALID;
import static gregapi.data.CS.ERR;
import static gregapi.data.CS.F;
import static gregapi.data.CS.T;
import gregapi.util.UT;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;

/**
 * @author Gregorius Techneticies
 */
public class CoverData {
	public short[] mIDs, mMetas, mVisuals, mValues;
	public boolean[] mVisualsToSync = new boolean[] {F,F,F,F,F,F};
	public NBTTagCompound[] mNBTs;
	public ICover[] mBehaviours = new ICover[6];
	public ITileEntityCoverable mTileEntity = null;
	
	public CoverData(ITileEntityCoverable aTileEntity) {
		mIDs = new short[6]; mMetas = new short[6]; mVisuals = new short[6]; mValues = new short[6]; mNBTs = new NBTTagCompound[6];
		mTileEntity = aTileEntity;
	}
	
	public CoverData(short[] aIDs, short[] aMetas, short[] aVisuals, short[] aValues, NBTTagCompound[] aNBTs, ITileEntityCoverable aTileEntity) {
		mVisuals = aVisuals; mValues = aValues; mNBTs = aNBTs;
		for (int i = 0; i < mNBTs.length;i++) if (mNBTs[i] != null && mNBTs[i].hasNoTags()) mNBTs[i] = null;
		setIDs(aIDs, aMetas);
		mTileEntity = aTileEntity;
	}
	
	public CoverData(ITileEntityCoverable aTileEntity, NBTTagCompound aNBT) {
		this( new short[] {aNBT.getShort("a"), aNBT.getShort("b"), aNBT.getShort("c"), aNBT.getShort("d"), aNBT.getShort("e"), aNBT.getShort("f")}
			, new short[] {aNBT.getShort("g"), aNBT.getShort("h"), aNBT.getShort("i"), aNBT.getShort("j"), aNBT.getShort("k"), aNBT.getShort("l")}
			, new short[] {aNBT.getShort("m"), aNBT.getShort("n"), aNBT.getShort("o"), aNBT.getShort("p"), aNBT.getShort("q"), aNBT.getShort("r")}
			, new short[] {aNBT.getShort("0"), aNBT.getShort("1"), aNBT.getShort("2"), aNBT.getShort("3"), aNBT.getShort("4"), aNBT.getShort("5")}
			, new NBTTagCompound[] {aNBT.getCompoundTag("s"), aNBT.getCompoundTag("t"), aNBT.getCompoundTag("u"), aNBT.getCompoundTag("v"), aNBT.getCompoundTag("w"), aNBT.getCompoundTag("x")}
			, aTileEntity);
	}
	
	public NBTTagCompound writeToNBT(NBTTagCompound aNBT, boolean aIncludeVisuals) {
		byte i = 0;
		if (mIDs[  i] != 0) {
			aNBT.setShort("a", mIDs[i]);
			if (mMetas[i] != 0) aNBT.setShort("g", mMetas[i]);
			if (mValues[i] != 0) aNBT.setShort("0", mValues[i]);
			if (mNBTs[i] != null && !mNBTs[i].hasNoTags()) aNBT.setTag("s", mNBTs[i]);
			if (mVisuals[i] != 0 && (aIncludeVisuals || (mBehaviours[i] != null && mBehaviours[i].needsVisualsSaved(i, this)))) aNBT.setShort("m", mVisuals[i]);
		}
		if (mIDs[++i] != 0) {
			aNBT.setShort("b", mIDs[i]);
			if (mMetas[i] != 0) aNBT.setShort("h", mMetas[i]);
			if (mValues[i] != 0) aNBT.setShort("1", mValues[i]);
			if (mNBTs[i] != null && !mNBTs[i].hasNoTags()) aNBT.setTag("t", mNBTs[i]);
			if (mVisuals[i] != 0 && (aIncludeVisuals || (mBehaviours[i] != null && mBehaviours[i].needsVisualsSaved(i, this)))) aNBT.setShort("n", mVisuals[i]);
		}
		if (mIDs[++i] != 0) {
			aNBT.setShort("c", mIDs[i]);
			if (mMetas[i] != 0) aNBT.setShort("i", mMetas[i]);
			if (mValues[i] != 0) aNBT.setShort("2", mValues[i]);
			if (mNBTs[i] != null && !mNBTs[i].hasNoTags()) aNBT.setTag("u", mNBTs[i]);
			if (mVisuals[i] != 0 && (aIncludeVisuals || (mBehaviours[i] != null && mBehaviours[i].needsVisualsSaved(i, this)))) aNBT.setShort("o", mVisuals[i]);
		}
		if (mIDs[++i] != 0) {
			aNBT.setShort("d", mIDs[i]);
			if (mMetas[i] != 0) aNBT.setShort("j", mMetas[i]);
			if (mValues[i] != 0) aNBT.setShort("3", mValues[i]);
			if (mNBTs[i] != null && !mNBTs[i].hasNoTags()) aNBT.setTag("v", mNBTs[i]);
			if (mVisuals[i] != 0 && (aIncludeVisuals || (mBehaviours[i] != null && mBehaviours[i].needsVisualsSaved(i, this)))) aNBT.setShort("p", mVisuals[i]);
		}
		if (mIDs[++i] != 0) {
			aNBT.setShort("e", mIDs[i]);
			if (mMetas[i] != 0) aNBT.setShort("k", mMetas[i]);
			if (mValues[i] != 0) aNBT.setShort("4", mValues[i]);
			if (mNBTs[i] != null && !mNBTs[i].hasNoTags()) aNBT.setTag("w", mNBTs[i]);
			if (mVisuals[i] != 0 && (aIncludeVisuals || (mBehaviours[i] != null && mBehaviours[i].needsVisualsSaved(i, this)))) aNBT.setShort("q", mVisuals[i]);
		}
		if (mIDs[++i] != 0) {
			aNBT.setShort("f", mIDs[i]); 
			if (mMetas[i] != 0) aNBT.setShort("l", mMetas[i]);
			if (mValues[i] != 0) aNBT.setShort("5", mValues[i]);
			if (mNBTs[i] != null && !mNBTs[i].hasNoTags()) aNBT.setTag("x", mNBTs[i]);
			if (mVisuals[i] != 0 && (aIncludeVisuals || (mBehaviours[i] != null && mBehaviours[i].needsVisualsSaved(i, this)))) aNBT.setShort("r", mVisuals[i]);
		}
		return aNBT;
	}
	
	public CoverData setIDs(short[] aIDs, short[] aMetas) {
		mIDs = aIDs; mMetas = aMetas;
		for (byte tSide : ALL_SIDES_VALID) if (mIDs[tSide] == 0) mBehaviours[tSide] = null; else mBehaviours[tSide] = CoverRegistry.get(mIDs[tSide], mMetas[tSide]);
		return this;
	}
	
	public CoverData set(byte aSide, ItemStack aStack) {
		return aStack == null ? set(aSide, (short)0, (short)0, null) : set(aSide, UT.Stacks.id(aStack), UT.Stacks.meta(aStack), aStack.getTagCompound());
	}
	
	public CoverData set(byte aSide, short aID, short aMeta, NBTTagCompound aNBT) {
		mIDs[aSide] = aID; mMetas[aSide] = aMeta;
		if (aID == 0) mBehaviours[aSide] = null; else mBehaviours[aSide] = CoverRegistry.get(aID, aMeta);
		mNBTs[aSide] = (NBTTagCompound)(aNBT==null||aNBT.hasNoTags()?null:aNBT.copy());
		return this;
	}
	
	public CoverData value(byte aSide, short aValue) {
		mValues[aSide] = aValue;
		return this;
	}
	
	public CoverData visual(byte aSide, short aVisual) {
		if (mVisuals[aSide] != aVisual) {mVisuals[aSide] = aVisual; mVisualsToSync[aSide] = T;}
		return this;
	}
	
	public ItemStack getCoverItem(byte aSide) {
		return mIDs[aSide] == 0 ? null : mBehaviours[aSide] == null ? UT.Stacks.make(mIDs[aSide], 1, mMetas[aSide], mNBTs[aSide]==null||mNBTs[aSide].hasNoTags()?null:mNBTs[aSide]) : mBehaviours[aSide].getCoverItem(aSide, this);
	}
	
	public boolean requiresSync() {
		return UT.Code.containsBoolean(T, mVisualsToSync);
	}
	
	public void resetSync() {
		for (int i = 0; i < mVisualsToSync.length; i++) mVisualsToSync[i] = F;
	}
	
	public void tickPre(long aTimer, boolean aIsServerSide) {
		try {for (byte tSide : ALL_SIDES_VALID) if (mBehaviours[tSide] != null) mBehaviours[tSide].onTickPre (tSide, this, aTimer, aIsServerSide);} catch(Throwable e) {e.printStackTrace(ERR); mTileEntity.setError("Cover Pre Tick - "  + (aIsServerSide?"Serverside: ":"Clientside: ") + e.getLocalizedMessage());}
	}
	
	public void tickPost(long aTimer, boolean aIsServerSide) {
		try {for (byte tSide : ALL_SIDES_VALID) if (mBehaviours[tSide] != null) mBehaviours[tSide].onTickPost(tSide, this, aTimer, aIsServerSide);} catch(Throwable e) {e.printStackTrace(ERR); mTileEntity.setError("Cover Post Tick - " + (aIsServerSide?"Serverside: ":"Clientside: ") + e.getLocalizedMessage());}
	}
}